import SiGL
import nanothreads
import random

from SiGL.vector import Vector
import eventnet.driver

class TyphoonEffector(object):
    def __init__(self, texture, sound, position):
        self.sound = sound
        self.images = (
                            SiGL.draw.Image(texture['cloud.png'], position=(0,0),color=[1,1,1,0.5]),
                            SiGL.draw.Image(texture['cloud.png'], position=(0.01,0.01),color=[1,1,1,0.6], scale=0.7),
                            SiGL.draw.Image(texture['cloud.png'], position=(0.02,0.02),color=[1,1,1,0.8], scale=1.0),
                            )
        self.graph = SiGL.graph.Graph()
        self.position = position
        self.graph.position = 0,0
        self.penalty = 3
        for image in self.images:
            self.graph.add(image)
        
        nanothreads.defer(self.execute)
        
        
    def execute(self):
        SiGL.actions.Spin(self.images[0],degrees=360,duration=10).execute()
        SiGL.actions.Spin(self.images[1],degrees=3600,duration=10).execute()
        SiGL.actions.Spin(self.images[2],degrees=36000,duration=10).execute()
        
        self.sound['wind.wav'].play()
        self.sound['wind.wav'].fadeout(5000)
        
        def post_damage(position):
            eventnet.driver.post('DAMAGE',position=position, radius=50, damage=0.3)
            
        for image in self.images:
            move_to = random.uniform(-0.01,0.01),random.uniform(-0.01,0.01)
            SiGL.actions.Move(image, position=move_to).execute()
            thread = SiGL.actions.FadeOut(image, duration=10).execute()
        
        distance = Vector(self.position).magnitude
        SiGL.actions.Move(self.graph, position=self.position, duration=distance,callback=post_damage).execute()
        nanothreads.register_exit(thread, self.terminate)
        
    def terminate(self):
        self.graph.container.remove(self.graph)
        
        

class LightningEffector(object):
    def __init__(self, texture, sound, position):
        self.sound = sound
        self.graph = SiGL.graph.Graph()
        self.position = position
        
        self.graph.position = 0,0
        self.penalty = 2
        self.image = SiGL.draw.Animation([texture['lightning1.png'],texture['lightning2.png']],fps=60)
        self.mark = SiGL.draw.Image(texture['scorch.png'],color=[1,1,1,0])
        self.mark.rotation = random.uniform(1,360)
        self.mark.scale = random.uniform(0.5,1.0)
        self.flash = SiGL.draw.Rectangle([(-5,5),(5,-5)],color=[1,1,1,0],position=(0,0))
        self.image.position = 0,0
        self.mark.position = position
        self.graph.add(self.mark)
        nanothreads.defer(self.execute)

    def execute(self):
        self.graph.add(self.image)
        self.graph.add(self.flash)
        diff = Vector((0,0)) - self.position
        distance = diff.magnitude * 0.1
        fth = SiGL.actions.FadeIn(self.flash, duration=0.05, amount=0.3).execute()
        def unflash():
            SiGL.actions.FadeOut(self.flash, duration=0.1).execute()
        nanothreads.register_exit(fth, unflash)
        self.sound['lightning.wav'].play()
        self.sound['lightning.wav'].fadeout(5000)
        self.image.rotation = diff.angle() + 90
        self.image.position = self.position.unit * 0.6
        dam = SiGL.actions.Move(self.image, position=self.position, duration=distance).execute()
        SiGL.actions.FadeOut(self.image).execute()
        SiGL.actions.Implode(self.image, duration=distance, amount=0.9).execute()
        thread = SiGL.actions.FadeIn(self.mark, duration=distance).execute()
        
        nanothreads.register_exit(dam, eventnet.driver.post, "DAMAGE",position=self.position, radius=3, damage=9)
        
        def fade():
            fth2 = SiGL.actions.FadeIn(self.flash, duration=0.05, amount=0.3).execute()
            th = SiGL.actions.FadeOut(self.mark, duration=2).execute()
            nanothreads.register_exit(th, self.graph.container.remove, self.graph)
            nanothreads.register_exit(fth2, unflash)
        nanothreads.register_exit(thread, fade)
        

class QuakeEffector(object):
    def __init__(self, texture, sound, position):
        self.texture = texture
        self.sound = sound
        self.position = position
        self.graph = SiGL.graph.Graph()
        self.graph.position = 0,0
        self.penalty = 5
        self.crater = SiGL.draw.Image(texture['crator.png'], position=position,scale=0.3,rotation=random.uniform(1,360))
        nanothreads.defer(self.execute)
        
    def execute(self):
        self.graph.add(self.crater)
        SiGL.actions.Explode(self.crater,duration=2.0).execute()
        self.sound['quake.ogg'].play()
        dam = SiGL.actions.Quake(self.container.graph,duration=5,delay=0.05, amount=0.02).execute()
        th = SiGL.actions.FadeOut(self.crater, duration=30).execute()
        nanothreads.register_exit(th, self.graph.container.remove, self.graph)
        nanothreads.register_exit(dam, eventnet.driver.post, "DAMAGE",position=self.position, radius=400, damage=4)
        


class MeteorEffector(object):
    def __init__(self, texture, sound, position):
        self.texture = texture
        self.sound = sound
        self.position = position
        self.graph = SiGL.graph.Graph()
        self.graph.position = 0,0
        self.penalty = 50
        start_pos = Vector(position) + (1,1)
        self.crater = SiGL.draw.Image(texture['crator.png'], position=position,scale=0.3,rotation=random.uniform(1,360))
        self.image = SiGL.draw.Image(texture['meteor.png'], position=start_pos, color=[1,1,1,0])
        self.shock = SiGL.draw.Image(texture['shockwave.png'], position=position, scale=0.1,color=[1,1,1,0])
        self.shadow = SiGL.draw.Image(texture['shadow.png'],position=position, color=[1,1,1,0.6])
        self.flash = SiGL.draw.Rectangle([(-5,5),(5,-5)],color=[1,1,1,0],position=(0,0))
        nanothreads.defer(self.execute)
        
    def nothing(self):
        pass
        
    def execute(self):
        self.graph.add(self.shadow)
        self.graph.add(self.image)
        self.graph.add(self.shock)
        self.graph.add(self.flash)
        self.sound['quake.ogg'].play()
        SiGL.actions.FadeIn(self.shadow,duration=7, amount=0.7).execute()
        SiGL.actions.FadeIn(self.image,duration=0.1).execute()
        SiGL.actions.Move(self.image, position=self.position, duration=15).execute()
        SiGL.actions.Implode(self.image, duration=15).execute()
        SiGL.actions.Implode(self.shadow, duration=15).execute()
        th = SiGL.actions.Spin(self.image, duration=15).execute()
        
        def cleanup():
            self.graph.container.remove(self.graph)
            
            
        def fade():
            th = SiGL.actions.FadeOut(self.flash, duration=1.0).execute()
            
            nanothreads.register_exit(th, cleanup)
            
        def finish():
            self.shock.color = [1,1,1,0.1]
            fine = SiGL.actions.FadeIn(self.flash, duration=3.0, amount=0.9).execute()
            SiGL.actions.Explode(self.shock, duration=0.2, size=10.0).execute()
            eventnet.driver.post("DAMAGE",position=self.position, radius=400, damage=25)
            nanothreads.register_exit(fine, fade)
        nanothreads.defer_for(11.99, self.sound['asteroid.ogg'].play)
        MeteorEffector.execute = MeteorEffector.nothing
        nanothreads.register_exit(th, finish)

        
